#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"
#define VENDOR_RQ_SET_LED    0
#define VENDOR_RQ_GET_SWITCH 1
#define VENDOR_RQ_GET_LIGHT  2

uint8_t in_c(uint8_t pin)
{
    // กำหนดให้ขาที่ระบุของพอร์ท C ทำหน้าที่เป็นอินพุท
    DDRC &= ~(1<<pin);
    // ดึงให้ขามีลอจิก 1 หากขาถูกปล่อยลอยไว้ 
    PORTC |= (1<<pin);
    // อ่านสถานะลอจิกของขา
    return ((PINC & (1<<pin))>>pin);    
} 

void out_c(uint8_t pin, uint8_t val)
{
    // กำหนดให้ขาที่ระบุของพอร์ท C ทำหน้าที่เป็นเอาท์พุท
    DDRC |= 1<<pin;
    // เซ็ตลอจิกของขาเป็น 1 ถ้า val ไม่ใช่ศูนย์ ไม่เช่นนั้นเซ็ตให้เป็น 0
    if (val)
        PORTC |= 1<<pin;
    else
        PORTC &= ~(1<<pin);
}

uint16_t read_adc(uint8_t channel)
{
    ADMUX = 0b01000000; // เลือกใช้แรงดันอ้างอิงจากขา AREF
    ADMUX |= channel;   // ตั้ง MUX ให้ชี้ไปยัง channel
    ADCSRA = 0b11000110;
    //         ||   -+-
    //         ||    |
    //         ||    +-> ปรับคล็อกของ ADC ให้เป็น 1/64 ของคล็อกระบบ
    //         |+->เริ่มแปลงค่าแอนะล็อกเป็นดิจิทัล
    //         |
    //         +-> Enable ADC

    while ((ADCSRA & (1<<ADSC))) // รอจนบิต ADSC (บิต 6) กลายเป็น 0
        ;
    return ADCL + ADCH*256;  // ผลลัพธ์ถูกเก็บอยู่ในรีจีสเตอร์ ADCL และ ADCH
}

usbMsgLen_t usbFunctionSetup(uchar data[8])
{
    usbRequest_t *rq = (void *)data;
    static uchar dataBuffer[2];  /* ข้อมูลนี้ต้องไม่ถูกเขียนทับหลังจาก usbFunctionSetup รีเทิร์น */
    //int i;
    usbMsgLen_t len = 64;
        
    /* ประมวลผลตามหมายเลขคำสั่งที่อยู่ใน bRequest */
    if (rq->bRequest == VENDOR_RQ_SET_LED)
    {
        uint8_t led_no  = rq->wValue.bytes[0];
        uint8_t led_val = rq->wValue.bytes[1];
        out_c(led_no, led_val);
        return 0;
    }
    else if (rq->bRequest == VENDOR_RQ_GET_SWITCH)
    {
        len = 1;        //cuz switch has value only 1 bit
        dataBuffer[0] = !in_c(3); /* กลับลอจิกเพื่อให้ 0 = ปล่อย 1 = กด */
        usbMsgPtr = dataBuffer;   /* ระบุว่าข้อมูลส่งกลับอยู่ใน dataBuffer */
        return len;                 /* ความยาวข้อมูลส่งกลับเท่ากับ 1 */
    }
    else if (rq->bRequest == VENDOR_RQ_GET_LIGHT)
    {
        len = 2;                        // return 2bytes of light level(uint16bit)
        DDRC &= 0b11101111;             // set PC4 to input
        static uint16_t volt = 0;
        volt = read_adc(4);             // light sensor @ PC4
        //for(i = 0; i< 10; i++) {
        //    dataBuffer[i] = (uchar)( ((1 << (9-i)) & volt) >> (9-1) );
        //}
        usbMsgPtr = &volt;
        //usbMsgPtr = read_adc(4);
        return len;
    }
    return 0;   /* ไม่รู้จักคำร้องขอ ไม่ส่งข้อมูลกลับ */
}

int main()
{
    PORTC |= 1 << 4; 
    usbInit();
    usbDeviceDisconnect();
    _delay_ms(300);     /* fake USB disconnect for > 250 ms */
    usbDeviceConnect();
    sei();              /* enable interrupts */
    while (1)           /* main event loop */
    {                
        usbPoll();      /* คำสั่งนี้ต้องถูกเรียกอย่างน้อยที่สุดทุก ๆ 50ms */
    }
    return 0;
}

